@@ -1,33 +1,32 @@
-import re
 import asyncio
 import logging
+import re
 import threading
 from collections import defaultdict
 
-from .common import ControlPacketType
-from .common import ConnectReasonCode
-from .common import SubackReasonCode
-from .common import UnsubackReasonCode
-from .common import DisconnectReasonCode
-from .common import PropertyIds
-from .common import MalformedPacketError
-from .common import PayloadReader
-from .common import unpack_connect
-from .common import pack_connack
-from .common import pack_publish
-from .common import unpack_publish
-from .common import unpack_subscribe
-from .common import pack_suback
-from .common import unpack_unsubscribe
-from .common import pack_unsuback
-from .common import pack_pingresp
-from .common import pack_disconnect
-from .common import unpack_disconnect
-from .common import format_packet
-from .common import format_packet_compact
 from .common import CF_FIXED_HEADER
 from .common import MAXIMUM_PACKET_SIZE
-
+from .common import ConnectReasonCode
+from .common import ControlPacketType
+from .common import DisconnectReasonCode
+from .common import MalformedPacketError
+from .common import PayloadReader
+from .common import PropertyIds
+from .common import SubackReasonCode
+from .common import UnsubackReasonCode
+from .common import format_packet
+from .common import format_packet_compact
+from .common import pack_connack
+from .common import pack_disconnect
+from .common import pack_pingresp
+from .common import pack_publish
+from .common import pack_suback
+from .common import pack_unsuback
+from .common import unpack_connect
+from .common import unpack_disconnect
+from .common import unpack_publish
+from .common import unpack_subscribe
+from .common import unpack_unsubscribe
 
 LOGGER = logging.getLogger(__name__)
 
@@ -132,7 +131,8 @@ class Client(object):
             if self._session.will_topic is not None:
                 if not self.is_normal_disconnection():
                     self._broker.publish(self._session.will_topic,
-                                         self._session.will_message)
+                                         self._session.will_message,
+                                         {})
 
                     if self._session.will_retain:
                         self._broker.add_retained_message(self._session.will_topic,
@@ -240,7 +240,7 @@ class Client(object):
         self.log_info('Client connected.')
 
     def on_publish(self, payload, flags):
-        topic, message, _ = unpack_publish(payload, (flags >> 1) & 3)
+        topic, message, properties = unpack_publish(payload, (flags >> 1) & 3)
 
         if is_wildcards_in_topic(topic):
             raise MalformedPacketError(f'Invalid topic {topic} in publish.')
@@ -251,7 +251,7 @@ class Client(object):
             else:
                 self._broker.remove_retained_message(topic)
 
-        self._broker.publish(topic, message)
+        self._broker.publish(topic, message, properties)
 
     def on_subscribe(self, payload):
         packet_identifier, _, subscriptions = unpack_subscribe(payload)
@@ -282,7 +282,7 @@ class Client(object):
         self._write_packet(pack_suback(packet_identifier, reasons))
 
         for topic, message in retained_messages:
-            self.publish(topic, message, True)
+            self.publish(topic, message, True, {})
 
     def on_unsubscribe(self, payload):
         packet_identifier, topics = unpack_unsubscribe(payload)
@@ -313,8 +313,8 @@ class Client(object):
 
         raise DisconnectError()
 
-    def publish(self, topic, message, retain=False):
-        self._write_packet(pack_publish(topic, message, retain, None))
+    def publish(self, topic, message, retain, properties):
+        self._write_packet(pack_publish(topic, message, retain, properties))
 
     def disconnect(self):
         self._write_packet(pack_disconnect(self._disconnect_reason))
@@ -542,13 +542,13 @@ class Broker(object):
     def remove_session(self, client_id):
         del self._sessions[client_id]
 
-    def  publish(self, topic, message):
+    def publish(self, topic, message, properties):
         """Publish given topic and message to all subscribers.
 
         """
 
         for session in self.iter_subscribers(topic):
-            session.client.publish(topic, message)
+            session.client.publish(topic, message, False, properties)
 
 
 class BrokerThread(threading.Thread):
